/* Compute the largest singular value of the matrix M and the associated singular vectors */

#include "svdPower.h"
#include <mkl_cblas.h>

void eigVecSymPower(double* A,int n, double* x0, double* xn) {
   int i;
   double* xoriginal=(double*)malloc(n*sizeof(double));
   memcpy(xoriginal,x0,n*sizeof(double));
   for (i = 0; i<MAX_ITER; ++i) {
      cblas_dsymv(CblasColMajor,CblasUpper,n,1.0,A,n,x0,1,0.0,xn,1);
      double invNorm = 1.0/cblas_dnrm2(n,xn,1);
      cblas_dscal(n,invNorm,xn,1);
      double theta=cblas_ddot(n,xn,1,x0,1);
      if ((1 - theta) < EPSILON_POWER) break;
      memcpy(x0,xn,n*sizeof(double));
   }
   if (i == MAX_ITER) memcpy(xn,xoriginal,n*sizeof(double));
   free(xoriginal);
}

/* Here, m >> n and M is a n x m matrix */
void svdPower2(double * M, int n, int m,double* d0,double* u, double* v) {
   int i;
   int sizeC=n*n;
   double* c = (double*)malloc(sizeC*sizeof(double));
   for (i = 0; i<sizeC; ++i) c[i]=0.0;
   cblas_dsyrk(CblasColMajor,CblasUpper,CblasNoTrans,n,m,1.0,M,n,0.0,c,n);
   eigVecSymPower(c,n,d0,u);
   free(c);
   cblas_dgemv(CblasColMajor,CblasTrans,n,m,1.0,M,n,u,1,0.0,v,1);
}

/* Here, m >> n and M is a n x m matrix */
void svdPower(double * M, int n, int m,double* d0,double* u, double* v) {
   int i;
   double* up=(double*)malloc(n*sizeof(double));
   cblas_dcopy(n,d0,1,up,1);
   cblas_dgemv(CblasColMajor,CblasTrans,n,m,1.0,M,n,up,1,0.0,v,1);
   for (i = 0; i<MAX_ITER; ++i) {
      cblas_dgemv(CblasColMajor,CblasNoTrans,n,m,1.0,M,n,v,1,0.0,u,1);
      double norm = cblas_dnrm2(n,u,1);
      cblas_dscal(n,1.0/norm,u,1);
      cblas_dgemv(CblasColMajor,CblasTrans,n,m,1.0,M,n,u,1,0.0,v,1);
      double theta=cblas_ddot(n,u,1,up,1);
      if ((1 - fabs(theta)) < EPSILON_POWER) break;
      cblas_dcopy(n,u,1,up,1);
   }
   free(up);
   if (i == MAX_ITER) {
      cblas_dcopy(n,d0,1,u,1);
      cblas_dgemv(CblasColMajor,CblasTrans,n,m,1.0,M,n,u,1,0.0,v,1);
   }
}


