/* timer.h */
/* KSVDRestore for denoising, inpainting, demosaicing and scale-up
   Copyright (C) 2007 Julien Mairal 

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
   MA  02110-1301, USA.
*/

/*!
 * \file
 *                Multiscale KSVD Denoising
 *
 *                      by Julien Mairal
 *                      julien.mairal@m4x.org
 *
 *                      File timer.h
 * \brief Contains the class Timer */

#ifndef TIMER_H
#define TIMER_H

#include <iostream>
#include <qdatetime.h>

using namespace std;

namespace KSVD {
   /// \brief Simple portable Timer 
   class Timer {

      public:
         /// Empty Constructor
         inline Timer() : _started(false) { };

         /// Start the timer
         inline void start() {
            _time.start();
            _cumul = 0.0;
            _started = true;
         };
         /// Stop the timer
         inline void stop() {
            if (!_started)
               cerr << "Timer not started" << endl;
            _elapsed = static_cast<double>(_time.elapsed())/1000 + _cumul;
            _started = false;
         };
         /// pause the timer
         inline void pause() {
            if (!_started)
               cerr << "Timer not started" << endl;
            _cumul += static_cast<double>(_time.elapsed())/1000;
            _started = false;
         };
         /// restart the timer
         inline void restart() {
            _time.start();
            _started = true;
         };

         /// return the number of elapsed seconds
         inline double secondsElapsed() const {
            return _elapsed;
         };
         /// print the number of elapsed seconds
         inline void printTimeElapsed() const {
            std::cout << "Number of seconds elapsed : ";
            std::cout << _elapsed << std::endl;
         };

      private:
         /// the QTTimer
         QTime _time;
         /// number of secondes elapsed
         double _elapsed;
         /// cumulative time
         double _cumul;
         /// started or not
         bool _started;
         /// copy-constructor is forbidden
         explicit Timer(const Timer& timer);
         /// operator = is forbidden
         Timer& operator=(const Timer& timer);
   };
};

#endif
